
require 'erb'
require 'rexml/document'

module Amrita2
 module ScalarData
    def amrita_value
      Amrita2::SanitizeSupport::sanitize_text(to_s)
    end
  end

  module DictionaryData
    def amrita_value(name)
      self.__send__(name)
    end
    
    def self.===(other)
      if other == nil
        true
      else
        super
      end
    end
  end

  module AmritaEnum
  end

  module NullObject
    def amrita_value(key=nil)
      nil
    end

    def each
    end
  end
end

class NilClass
  include Amrita2::NullObject
end

class FalseClass
  include Amrita2::NullObject
end

class String
  include Amrita2::ScalarData
  def amrita_value
    Amrita2::SanitizeSupport::sanitize_text(self)
  end
end

class Integer
  include Amrita2::ScalarData
end

class REXML::Element
  def amrita_value
    Amrita2::SanitizedString[to_s]
  end
end

class Array
  include Amrita2::AmritaEnum
end

class Range
  include Amrita2::AmritaEnum
end

class Hash
  include Amrita2::DictionaryData
  def amrita_value(name)
    self[name]
  end
end

class Struct
  include Amrita2::DictionaryData
end

class Binding
  include Amrita2::DictionaryData

  def amrita_value(name)
    eval "begin; #{name}; rescue(NameError); @#{name};end;", self
  end
end

module Amrita2
  FilterMethods = [
    :filter_element,
    :parse_element,
    :parse_node,
    :setup_type_renderer,
    :generate_dynamic_element,
    :define_element_method,
    :loop_check_code,
    :method_body_code,
    :value_filter_code,
    :renderer_code,
    :element_render_code
  ]


  module Filters
    class Base
    end
  end

  module Renderers
  end

  module SanitizeSupport # :nodoc:

    # Amrita2 sanitize anything except for SanitizedString
    # If you want to sanitize yourself and don't want to Amrita2 sanitize your object,
    # pass SanitizedString[x] as model data.
    class SanitizedString < String
      def SanitizedString::[](s)
        new(s.to_s).freeze
      end

      def amrita_value
        self
      end

      def to_s
        self
      end

      def *(n)
        SanitizedString[super]
      end

      def inspect
        %[SanitizedString[#{super}]]
      end
    end

    # This module provide methods for avoid XSS vulnerability
    # taken from IPA home page(Japanese)
    # http://www.ipa.go.jp/security/awareness/vendor/programming/a01_02.html
    NAMECHAR = '[-\w\d\.:]'
    NAME = "([\\w:]#{NAMECHAR}*)"
    NOT_REFERENCE = "(?!#{NAME};|&#\\d+;|&#x[0-9a-fA-F]+;)" # borrowed from rexml
    AMP_WITHOUT_REFRENCE = /&#{NOT_REFERENCE}/
    # escape &<>
    def self.sanitize_text(text)
      s = text.dup
      s.gsub!(AMP_WITHOUT_REFRENCE, '&amp;')
      s.gsub!("<", '&lt;')
      s.gsub!(">", '&gt;')
      s
    end
  end
  include SanitizeSupport

  module Util
    module OptionSupport
      attr_reader :opt
      def parse_opt(*args)
        @opt = {}
        hash = args.last
        if hash.kind_of?(Hash)
          @opt.merge!(hash)
          args.pop
        end
        
        args.each do |key|
          @opt[key] = key
        end
        @opt
      end
      
      def method_missing(sym, *args, &block)
        if @opt and @opt.has_key?(sym)
          @opt[sym]
        else
          super
        end
      end
    end
  end
  include Util
  
  module Runtime # :nodoc:
    def amrita_set_context_value(v)
      Thread::current[:amrita_context_value] = v
    end
    
    def amrita_get_context_value
      Thread::current[:amrita_context_value]
    end
    
    def new_element(tag, attrs={})
      ret = REXML::Element.new(tag.to_s)
      attrs.each do |k,v|
        ret.attributes[k.to_s] = v.to_s
      end
      ret
    end
    
    def start_tag(e, out="")
      out << "<#{e.expanded_name}"
      e.attributes.each_attribute do |attr|
        next if attr.value == ""
        out << " "
        # out << "#{attr.name}='#{attr.value.amrita_sanitize_as_attribute}'"
        attr.write(out)  
      end unless e.attributes.empty?
      out << ">"
      out
    end

    def end_tag(e, out="")
      out << "</#{e.expanded_name}>"
      out
    end
  end

  class CodeGenerator
    def initialize
      @iehack = true
      @lines = []
      @indent = 0
      init_strbuf
      @module_stack = [CGModule.new('')]
      @current_stream = "__stream__"
    end

    def init_strbuf
      @strbuf = ""
    end

    def code(line)
      flush
      @lines << [@indent, line]
    end
    
    def comment(comments)
      comments.each do |c|
        @lines << [@indent, "# #{c}"]
      end
    end
    
    def result
      flush
      m = @module_stack.shift
      if m
        m.constants.each do |name, defs|
          code("#{name} = #{defs}")
        end
      end
      @lines.collect do |indent, l|
        "  " * indent + l
      end.join("\n")
    end

    def put_constant(c)
      @strbuf.concat c.to_s
    end

    def flush
      if @strbuf.size > 0
        @lines << [@indent, "#{@current_stream}.concat(#{@strbuf.inspect})"]
        init_strbuf
      end
    end

    def put_rexml_node(node)
      s = ""
      node.write(s, -1, false, @iehack)
      put_constant s
    end

    def level_up
      @indent += 1
      yield
      flush
      @indent -= 1
    end

    def if_(cond, &block)
      code("if #{cond}") 
      level_up do 
        yield
      end
      code("end") 
    end

    def case_(obj, &block)
      code("case #{obj}") 
      yield
      code("end") 
    end

    def when_(obj, &block)
      code("when #{obj}") 
      level_up do
        yield
      end
    end

    def else_(&block)
      code("else") 
      level_up do
        yield
      end
    end

    def with_stream
      code("#@current_stream = '' ")
      yield
      code("#@current_stream")
    end

    def def_method(name, *args)
      if args.size > 0
        code("def #{name}(#{args.join(',')})")
      else
        code("def #{name}")
      end
      level_up do
        with_stream do
          yield
          flush
        end
      end
      code("end")
    end

    def put_static_element(e)
      attr = e.attributes.collect do |name, value|
        "#{name} = #{value.inspect}"
      end
      if attr.size > 0
        put_constant("<#{e.name} #{attr.join(' ')}>") 
      else
        put_constant("<#{e.name}>")
      end
      yield
      put_constant("</#{e.name}>")
    end

    def put_expression(exp)
      code("#{@current_stream}.concat((#{exp}).to_s)")
    end

    def put_string_expression(exp)
      code("#{@current_stream}.concat(#{exp})")
    end

    def define_class(name, &block)
      define_module_or_class("class", name, &block)
    end

    def define_module_or_class(typ, name, &block)
      @module_stack.unshift CGModule.new(name)
      code("#{typ} #{name}")
      level_up do
        yield
        m = @module_stack.shift
        m.end_of_module(self)
      end
      code("end")
    end

    def define_constant(const_def)
      @module_stack.first.define_constant(const_def)
    end

    class CGModule # :nodoc:
      attr_reader :name
      attr_reader :constants
      attr_reader :methods

      def initialize(name)
        @name = name 
        @constants = []
        @methods = []
      end

      def define_constant(const_def)
        name = nil
        @constants.each do |n, cdef|
          if cdef == const_def
            name = n
            break                       
          end
        end
        unless name
          name = "C#{sprintf("%03d", @constants.size)}"
          @constants << [name, const_def]
        end
        name
      end

      def define_method(vars, &block)
        name = "m#{sprintf("%03d", @methods.size)}"
        @methods << [name, vars, block]
        name
      end

      def end_of_module(cg)
        @constants.each do |name, defs|
          cg.code("#{name} = #{defs}")
        end
        @methods.each do |name, vars, body|
          cg.define_method(name, vars) do
            body.call
          end
        end
      end
    end
  end

  class BaseNode
    attr_reader :parent

    def initialize(parent)
      @parent = parent
    end
    
    def render_me(cg)
    end

    def module_src(cg)
    end
    
    def root
      parent.root
    end

    def parent_de
      parent
    end

    def dynamic?
      false
    end
  end

  class StaticNode < BaseNode
    def initialize(parent, node)
      super(parent)
      @node = node
    end

    def render_me(cg)
      cg.put_string_expression(cg.define_constant(@node.to_s.inspect))
    end
  end

  class ErbNode < BaseNode
    include Util
    attr_reader :node
    def initialize(parent, node)
      super(parent)
      @node = node
      @erb_trim_mode = nil
    end

    def dynamic?
      true
    end

    def render_me_new(cg)
      src = @node.to_s
      erb = ERB.new(%[#{with_context_value_erb(src).inspect}, nil, #{@erb_trim_mode.inspect}])
      puts erb.src.inspect
      erb_src = cg.define_constant(erb.src.inspect)
      cg.code("amrita_set_context_value($_)")
      cg.put_string_expression %[eval(#{erb_src},__binding__)]
      cg.code("$_ = amrita_get_context_value")
    end
    
    def render_me(cg)
      src = @node.to_s
      src.each do |s|
        cg.code("# #{s.chomp}")
      end
      erb = cg.define_constant %[ERB.new(#{with_context_value_erb(src).inspect}, nil, #{@erb_trim_mode.inspect})]
      cg.code("amrita_set_context_value($_)")
      cg.put_string_expression %[#{erb}.result(__binding__)]
      cg.code("$_ = amrita_get_context_value")
    end

    private
    def with_context_value_erb(src)
      [
        "<% $_ = amrita_get_context_value %>",
        src,
        "<% amrita_set_context_value($_) %>",
      ].join(" ")
    end
  end
  
  class ParentNode < BaseNode
    attr_reader :element, :parent, :children
    def initialize(parent, element)
      super(parent)
      @element = element
      @children = parent_de.parse_element(@element)
    end

    def dynamic?
      children.any? { |c| c.dynamic? }
    end
    
    def has_dynamic?
      children.any? { |c| c.dynamic? }
    end

    def each(&block)
      children.each(&block)
    end
    
    def module_src(cg)
      each do |c|
        c.module_src(cg)
      end
    end

  end

  class CompoundElement < ParentNode
    def render_me(cg)
      @parent.element_render_code(cg, @element) do
        each do |c|
          c.render_me(cg)
        end
      end
    end
  end

  class DynamicElement < ParentNode
    include Util
    include OptionSupport
    
    attr_reader :parent, :name, :children, :filter, :renderers, :element, :attrs
    def self.delegate_to_filter(name)
      module_eval <<-END
        def #{name}(*args,&block)
          @filter.#{name}(self, *args,&block)
        end
      END
    end

    FilterMethods.each do |m|
      delegate_to_filter(m)
    end

    def initialize(parent, name, element, filters=[])
      @parent,@name,@element = parent, name, element
      parse_opt(root.opt)
      @attrs = {}
      delete_am_attrs
      setup_filter(filters)
      @element = filter_element(@element)
      @renderers = []
      super(parent, element)
      
      setup_type_renderer
    end

    def [](key)
      @attrs[key]
    end

    def parent_de
      self
    end
    
    def dynamic?
      true
    end

    def setup_filter(filters)
      #default_filter = @parent ? @parent.filter : DefaultFilter.new
      default_filter =  DefaultFilter.new
      default_filter.parse_opt(opt)
      if filters.size > 0
        @filter = current = filters.first
        for f in filters do
          current.next_ = f
          current = f
        end
        current.next_ = default_filter
      else
        @filter = default_filter
      end
    end

    def class_name
      if name
        "XX" + name.capitalize
      else
        "XX%d" % [object_id.abs]
      end
    end

    def instance_name
      class_name + "Instance"
    end

    def compile(cg = CodeGenerator.new)
      define_element_method(cg) do
        cg.code("$_ = value")
        loop_check_code(cg)
        method_body_code(cg)
      end
    end

    def render_me(cg, n = name)
      if (n)
        cg.put_string_expression("#{instance_name}.render_with($_.amrita_value(:#{n}), __binding__)")
      else
        cg.put_string_expression("#{instance_name}.render_with($_, __binding__)")
      end
    end

    def module_src(cg)
      cg.define_class(class_name) do
        cg.code <<-END
          include Amrita2
          include Amrita2::Runtime
        END
        compile(cg)
        super
      end
      cg.code("#{instance_name} = #{class_name}.new")
    end

    def am_for_value
      @attrs[am_for.intern]
    end
    
    def am_skipif_value
      @attrs[am_skipif.intern]
    end

    def am_v_value
      @attrs[am_v.intern]
    end
    
    private
    
    def delete_am_attrs
      [
        am_src,
        am_filter,
        am_skipif,
        am_for,
        am_v,
      ].each do |key_s|
        key = key_s.intern
        @attrs[key] = @element.attributes[key_s]
        @element.delete_attribute(key_s)
      end
    end
  end

  class RootElement < DynamicElement
    include Filters
    def initialize(doc, opt={}, &block)
      parse_opt(opt)
      if block_given?
        @filter_proc = block
      else
        @filter_proc = proc do |e, src, filters| 
        end
      end
      e = doc.root
      filters_src = e.attributes[am_filter]
      filters = compile_filters(e, nil, filters_src)
      super(nil, nil, e, filters)
    end

    def root
      self
    end

    def compile(cg)
      cg.code <<-END
        include Amrita2
        include Amrita2::Runtime
      END
      super(cg)
      @children.each do |c|
        c.module_src(cg)
      end
    end

    def compile_filters(element, name, *args)
      src = args.compact.join('|')
      #filters = src.split('|').collect { |f| parse_filter(f) }
      filters = parse_filter(src)
      @filter_proc.call(element, name, filters)
      filters
    end
    
    def parse_filter(src)
      src.gsub!("\n", " ")
      case src
      when "",nil
        []
      else
        case ret = eval(src)
        when Class
          [ret.new]
        when Filters::Base
          [ret]
        when Array
          ret
        else
          raise "unknown Filter type #{ret.class}"
        end
      end
    end

    def parse_filter_a(f)
      case f = eval(f)
      when Class
        f.new
      else
        f
      end
    end
  end

  class DefaultFilter < Filters::Base
    include Renderers
    include Filters
    include Util::OptionSupport

    def filter_element(de, element)
      element
    end

    def parse_element(de, element)
      element.collect do |node|
        de.parse_node(node)
      end
    end

    def parse_node(de, node)
      case node
      when REXML::Element
        de.generate_dynamic_element(node) || CompoundElement.new(de, node)
      when REXML::CData
        ErbNode.new(de, node)
      when REXML::Text
        StaticNode.new(de, node)
      end
    end

    def generate_dynamic_element(de, e)
      src = e.attributes[de.am_src]
      filters_src = e.attributes[de.am_filter]
      if src
        name, fs = src.split('|', 2)
        filters = de.root.compile_filters(e, name, fs, filters_src)
        DynamicElement.new(de, name , e, filters)
      elsif filters_src
        filters = de.root.compile_filters(e, nil, filters_src)
        DynamicElement.new(de, nil , e, filters)
      else
        nil
      end
    end



    def setup_type_renderer(de)
      if de.has_dynamic?
        de.renderers << HashRenderer.new unless de.renderers.find {|r| r.kind_of?(HashRenderer) }
      else
        de.renderers << ScalarRenderer.new
      end
      #de.renderers << ArrayRenderer.new
      de.renderers << ElseRenderer.new
    end

    def define_element_method(de, cg, &block)
      cg.def_method("render_with", "value", "__binding__", "__cnt__ = -1 ", &block)
    end

    def loop_check_code(de, cg)
      cg.if_ "$_.kind_of?(AmritaEnum) and not $_.kind_of?(ScalarData)" do
        cg.code("$_.each_with_index  do |v, i| ")
        cg.level_up do 
          cg.put_string_expression("render_with(v, __binding__, i) ")
        end
        cg.code("end")
        cg.code("return __stream__")
      end
    end

    def method_body_code(de, cg)
      de.value_filter_code(cg, "value")
      #cg.code("p #{de.element.name.inspect}, $_, stream")
      de.renderer_code(cg, de.element)
      #cg.code("p #{de.element.name.inspect}, $_, stream")
    end

    def value_filter_code(de, cg, value_name)
    end

    def renderer_code(de, cg, element)
      r = de.renderers
      case r.size 
      when 0
      when 1
        r.first.generate_body(cg, de, element)
      else
        cg.case_("$_") do
          r.each do |r1|
            r1.generate_when(cg) do
              r1.generate_body(cg, de, element)
            end
          end
        end
      end
    end

    def element_render_code(de, cg, element, &block)
      if (element.name == 'span' && element.attributes.size == 0)
        yield
      else
        cg.put_static_element(element, &block)
      end
    end
  end

  module Renderers
    class RendererBase
      def initialize(type_name)
        @type_name = type_name
      end

      def generate_when(cg)
        cg.when_(@type_name) do
          yield
        end
      end
    end

    class ScalarRenderer < RendererBase
      def initialize
        super("ScalarData")
      end

      def generate_body(cg, de, element)
        de.element_render_code(cg, element) do
          cg.put_string_expression("$_.amrita_value")
        end
      end
    end

    class NilRenderer < RendererBase
      def initialize
        super("nil")
      end

      def generate_body(cg, de, element)
      end
    end

    class TrueRenderer < RendererBase
      def initialize
        super("true")
      end

      def generate_body(cg, de, element)
        cg.put_rexml_node(element)
      end
    end

    class HashRenderer < RendererBase
      def initialize
        super("DictionaryData")
      end


      def generate_body(cg, de, element)
        de.element_render_code(cg, element) do
          de.children.each do |c|
            c.render_me(cg)
          end
        end
      end
    end

    class ElseRenderer < RendererBase
      def initialize
        super("Object")
      end

      def generate_when(cg)
        cg.else_ do
          yield
        end
      end

      def generate_body(cg, de, element)
        de.element_render_code(cg, element) do
          cg.put_expression("$_.amrita_value")
        end
      end
    end
  end
  include Renderers

  module Filters
    include Renderers
    class Base
      attr_accessor :next_
      def self.filter_method(*m)
        m.each do |name|
          module_eval <<-END
            def #{name}(*args,&block)
              next_.#{name}(*args,&block)
            end
          END
        end
      end

      FilterMethods.each do |m|
        filter_method(m)
      end

      def self.inherited(cls)
        super
        def cls.[](*args, &block)
          new(*args, &block)
        end

        def cls.|(other)
          me = self.new
          case other
          when Class
            [me, other.new]
          when Filters::Base
            [me, other]
          when Array
            [me] + other
          else
            raise "unknown Filter type #{other.class}"
          end
        end
      end
      
      def |(other)
        me = self
        case other
        when Class
          [me, other.new]
        when Filters::Base
          [me, other]
        when Array
          [me] + other
        else
          raise "unknown Filter type #{other.class}"
        end
      end
      def parse_filter_a(f)
        case f = eval(f)
        when Class
          f.new
        else
          f
        end
      end
    end

    class Attr < Base
      class Renderer < Renderers::RendererBase
        include Util
        include OptionSupport
        def initialize(*args)
          parse_opt(*args)
          super("Amrita2::DictionaryData")
        end

        def generate_body(cg, de, element)
          body_key = nil
          unless de.has_dynamic?
            body_key = @opt[:body] || :body
            cg.code("body = $_.amrita_value(:#{body_key})")
          end
          if @opt.size > 0
            cg.code("a = {}")
            @opt.each do |key, v|
              next if key == :body
              cg.code("a[:#{key}] = $_.amrita_value(:#{v})")
            end
          else
            cg.code("a = $_ ")
          end
          
          a = { }
          element.attributes.each do |k, v|
            a[k] = v
          end
          cg.code("e = new_element(#{element.name.inspect}, #{a.inspect}.merge(a))")
          if de.has_dynamic?
            body_for_dynamic(cg, de, element)
          else
            body_for_static(cg)
          end
        end

        def body_for_static(cg)
          cg.if_("body") do
            cg.put_string_expression("start_tag(e)")
            cg.put_string_expression("body.amrita_value")
            cg.put_string_expression("end_tag(e)")
            cg.else_ do
              cg.put_string_expression("e.to_s")
            end
          end
        end
        
        def body_for_dynamic(cg, de, element)
          cg.put_string_expression("start_tag(e)")
          de.children.each do |c|
            c.render_me(cg)
          end
          cg.put_string_expression("end_tag(e)")
        end
      end

      def initialize(*args)
        @args = args
      end

      def setup_type_renderer(de)
        de.renderers.clear
        de.renderers << Renderer.new(*@args)
      end
    end


    class Default < Base
      def initialize(default_value_src)
        @default_value_src = default_value_src
      end

      def value_filter_code(de, cg, value)
        cg.code("$_ = $_ || #{@default_value_src} ")
        super
      end
    end
    
    class Repeat < Base
      def initialize(cnt)
        @cnt = cnt.to_i
      end

      def value_filter_code(de, cg, value)
        cg.code("$_ = $_ * #@cnt  ")
        super
      end
    end

    class Format < Base
      def initialize(format)
        @format = format
      end
      def value_filter_code(de, cg, value)
        cg.code("$_ = #{@format.inspect} % $_")
        super
      end
    end
    
    class AcceptData < Base
      def initialize(*types)
        @types = types
      end

      def setup_type_renderer(de)
        nil_processed = true_processed = hook_processed = false
        @types.each do |t|
          case t
          when nil, NilClass
            de.renderers << NilRenderer.new unless nil_processed
            nil_processed = true
          when true, TrueClass
            de.renderers << TrueRenderer.new unless true_processed
            true_processed = true
          when :hook
            de.renderers << Hook::Renderer.new unless hook_processed
            hook_processed = true
          else
            raise ArgumentError,"unknown type for AcceptData #{t.inspect}"
          end
        end
        super
      end
    end

    class NoSanitize < Base
      def value_filter_code(de, cg, value)
        super
        cg.code("$_ = SanitizedString[$_]")
      end
    end

    class InlineRuby < Base
      include Runtime
      include Renderers
      include Util
      
      def setup_type_renderer(de)
        de.renderers << HashRenderer.new unless de.renderers.find {|r| r.kind_of?(HashRenderer) }
        super
      end
      
      def generate_dynamic_element(de, e)
        a = e.attributes
        ret = super
        return ret if ret

        # check if element has any of inlineruby attributes
        if  %w(am_skipif am_for am_v).any? { |k| a[de.__send__(k)] }
          filters_src = e.attributes[de.am_filter]
          filters = de.root.compile_filters(e, nil, filters_src)
          DynamicElement.new(de, nil , e, filters)
        else
          nil
        end
      end

      def loop_check_code(de, cg)
        foreach_ = de.am_for_value
        if foreach_
          cg.if_("__cnt__ < 0") do
            cg.code("$_ = eval(#{foreach_.inspect}, __binding__)")
            super
          end
        else
          super
        end
      end

      def value_filter_code(de, cg, value)
        cond = de.am_skipif_value
        value = de.am_v_value
        
        if cond
          cg.code("amrita_set_context_value($_)")
          cg.code("return '' if eval(#{with_context_value_expression(cond).inspect},__binding__)")
        end

        if value
          cg.code("amrita_set_context_value($_)")
          cg.code("$_ = eval(#{with_context_value_expression(value).inspect},__binding__)")
        end
        super
      end
      
      private
      def with_context_value_expression(src)
        [
          "$_ = amrita_get_context_value",
          "ret = (#{src})",
          "amrita_set_context_value($_)",
          "ret"
        ].join(";")
      end
    end
    
    class NVar < Filters::Base
      def initialize(*names)
        @names = names
      end
      
      def renderer_code(de, cg, element)
        make_tupple_code(cg) if @names.size > 0
        replace_args(element)
        s = ""
        element.write(s, -1, false, false)
        cg.put_string_expression(%["#{s}"])
      end
      
      private
      
      def make_tupple_code(cg)
        init_code = @names.collect do |n|
          "$_.amrita_value(:#{n})"
        end.join(",")
        cg.code("$_ = Tuple[#{init_code}]")
      end

      def replace_args(element)
        element.attributes.each do|k,v|
          element.attributes[k] = replace_attr_args(v)
        end
        element.each_child do |c|
          case c
          when REXML::Text
            c.value = replace_text_args(c.value) if c.value
          when REXML::Element
            replace_args(c)
          end
        end
      end

      def replace_attr_args(s)
        s.gsub(/(.)?\$(\d)/) do |ss|
          if $1 == "$"
            "$#$2"
          else
            $1.to_s + '#{$_[' + ($2.to_i-1).to_s + ']}'
          end
        end
      end

      def replace_text_args(s)
        s.gsub(/(.)?\$(\d)/) do |ss|
          if $1 == "$"
            "$#$2"
          else
            $1.to_s + '#{$_[' + ($2.to_i-1).to_s + '].amrita_value}'
          end
        end
      end
    end
    
    class NVarForAttr < Filters::Base
      def initialize(*names)
        @names = names
      end
      
      def renderer_code(de, cg, element)
        make_tupple_code(cg) if @names.size > 0
        super
      end
      
      def element_render_code(de, cg, element, &block)
        if (element.name == 'span' && element.attributes.size == 0)
          yield
        else
          replace_args(element)
          if element.children.size > 0
            element.delete_if { true }
            s = element.to_s.sub(%[/>], ">")
            cg.put_string_expression(%["#{s}"])
            yield
            cg.put_constant("</#{element.name}>")
          else
            cg.put_string_expression(%["#{element.to_s}"])
          end
        end
      end
      
      private
      
      def make_tupple_code(cg)
        init_code = @names.collect do |n|
          "$_.amrita_value(:#{n})"
        end.join(",")
        cg.code("a = [#{init_code}]")
      end

      def replace_args(element)
        element.attributes.each do|k,v|
          element.attributes[k] = replace_attr_args(v)
        end
      end
      def replace_attr_args(s)
        s.gsub(/\$(\d)/) do |ss|
          '#{a[' + ($1.to_i-1).to_s + ']}'
        end
      end
    end
    
    class Trace < Filters::Base
      def filter_element(de, element)
        de.tracer << "befor macro: #{ element.to_s.inspect}" if de.tracer 
        new_element = super
        de.tracer << "after macro: #{ new_element.to_s.inspect}" if de.tracer 
        new_element
      end
      
      def value_filter_code(de, cg, value)
        cg.code(%[Tracer << "%s:%s:%s" % [#{de.element.inspect.inspect},"in", $_.inspect] ])
        super
      end
      
      def method_body_code(de, cg)
        super
        cg.code(%[Tracer << "%s:%s:%s" % [#{de.element.inspect.inspect},"out", __stream__] ])
      end
    end
  end

  class TemplateText
    include Amrita2
    include SanitizeSupport
    include Util
    include OptionSupport
    include Filters
    
    attr_accessor :inline_ruby, :amrita_prefix
    attr_reader :amrita_src, :tracer

    def initialize(text, *opts, &block)
      parse_opt(*opts)
      @text = text
      @setuped = false
      @filter_proc = block
      @text_domain = nil
      @inline_ruby = true
      @amrita_prefix = @opt[:amrita_prefix] || "am:"
      @preprocessor = PreProcessor.new
    end

    def amrita_src=(v)
      @opt[:amrita_src] = v
    end
    
    def amrita_filter=(v)
      @opt[:amrita_filter] = v
    end

    def set_trace(io_or_type, &block)
      @tracer = Tracer.new(io_or_type, &block)
    end

    def setup
      setup_opt
      cg = CodeGenerator.new
      setup_filter_proc
      @text.sub!(%r[<[^\?\!]]) { |m| "<root>" + m }
      @text = @text + "</root>"
      t = @preprocessor.process(@text)
      @doc = REXML::Document.new t
      compile(cg, @doc)
      @setuped = true
      cg.result
    ensure
      @tracer.code(cg) if @tracer and @tracer.code_trace?
    end

    def render_with(value, b=nil)
      setup unless @setuped
      unless b
        if value.kind_of?(Binding)
          b = value
        else
          b = TOPLEVEL_BINDING
        end
      end

      ret = ""
      
      if @opt[:process_xhtml]
        ret << @doc.xml_decl.to_s << "\n" if @doc.xml_decl
        ret << @doc.doctype.to_s << "\n"  if @doc.doctype
      end
      ret << @cls.new.render_with(value, b)
      ret.gsub!("<root>", "")
      ret.gsub!("</root>", "")
      SanitizedString[ret]
    end

    private
    def setup_opt
      @opt[:am_src] = @opt[:amrita_src] || @amrita_prefix + "src"
      @opt[:am_filter] = @opt[:amrita_filter] || @amrita_prefix + "filter"
      @opt[:am_skipif] = @opt[:amrita_skipif] || @amrita_prefix + "skipif"
      @opt[:am_for] = @opt[:amrita_for] || @amrita_prefix + "for"
      @opt[:am_v] = @opt[:amrita_v] || @amrita_prefix + "v"
      @opt[:tracer] = @tracer
    end
    
    def compile(cg, doc)
      @root = Amrita2::RootElement.new doc, opt, &@filter_proc
      @root.text_domain = @text_domain if @text_domain
      @root.compile(cg)

      @cls = Class.new
      @cls.module_eval cg.result
      @cls.const_set(:Tracer, @tracer) if @tracer
    rescue SyntaxError
      raise RuntimeError,$!
    end
    
    def setup_filter_proc
      old_proc = @filter_proc
      @filter_proc = proc do |e, name ,filters|
        filters.unshift Trace.new if @tracer and @tracer.auto_trace?
        filters.unshift InlineRuby.new if @inline_ruby
        old_proc.call(e, name ,filters) if old_proc
      end
    end

    class Tracer
      def initialize(io_or_type, &block)
        @auto_trace = false
        @trace_proc = block
        case io_or_type
        when :code, :element, :all
          @type = io_or_type          
          raise "need block" unless @trace_proc
        when :all_element
          @type = :element
          @auto_trace = true
          raise "need block" unless @trace_proc
        else
          if io_or_type.respond_to?(:<<)
            @type = :all
            @auto_trace = true
            @trace_proc = proc do |msg|
              io_or_type << msg << "\n"
            end
          else
            raise "unknown type for trace #{io_or_type.inspect}"
          end
        end
      end

      def auto_trace?
        @auto_trace
      end

      def code_trace?
        @type == :code or @type == :all
      end

      def code(cg)
        @trace_proc.call(cg.result) if code_trace?
      end

      def <<(msg)
        @trace_proc.call(msg) if @type == :all or @type == :element
      end
    end
  end

  class PreProcessor
    CDataStart = %r[(.*?)(<!\[CDATA\[)(.*)]m
    CDataEnd = %r[(.*?)(\]\]>)(.*)]m
    def initialize
    end

    def process(s)
      parse(s)
    end

    private
    def parse(s)
      if s =~ CDataStart
        out_of_cdata, cdata, in_cdata = $1, $2, $3
        process_out(out_of_cdata) + cdata + parse_in_cdata(in_cdata)
      else
        process_out(s)
      end
    end

    def process_out(s, nest=0)
      if s =~ %r[\A(.*?)(<%|%>)(.*)]m
        case $2
        when "<%"
          if nest == 0
            "#{$1}<![CDATA[<%#{process_out($3, nest+1)}"
          else
            "#{$1}<%#{process_out($3, nest+1)}"
          end
        when "%>"
          if nest == 1
            "#{$1}%>]]>#{process_out($3, nest-1)}"
          else
            "#{$1}%>#{process_out($3, nest-1)}"
          end
        else
          raise "can't happen"
        end
      else
        s
      end
    end

    def parse_in_cdata(s)
      if s =~ CDataEnd
        in_cdata, cdata_end, out_of_cdata = $1, $2, $3
        in_cdata + cdata_end + parse(out_of_cdata)
      else
        raise "end of cdata not found in #{s}"
      end
    end
  end
  
  class Tuple < Array # :nodoc:
    include ScalarData
    def self.[](*args)
      self.new(args)
    end
  end

  class Hook
    attr_reader :stream, :cnt, :element_s

    def initialize(&block)
      @hook_proc = block
    end
    
    def call(stream, cnt, element_s,&block)
      @stream = stream
      @cnt = cnt
      @element_s = element_s
      @render_me_proc = block
      instance_eval(&@hook_proc)
    end

    def render_me_with(it)
      stream << @render_me_proc.call(nil, it)
    end
    
    def render_child(name, it)
      stream << @render_me_proc.call(name, it)
    end

    class Renderer < Renderers::RendererBase
      def initialize
        super("Hook")
      end

      def generate_body(cg, de, element)
        cg.code("e = #{ cg.define_constant(element.to_s.inspect)}")
        cg.code("$_.call(__stream__, __cnt__, e) do |name, it| ")
        cg.level_up do 
          cg.case_('name') do
            cg.when_("nil") do 
              cg.code("render_with(it, __binding__)")
            end
            de.children.each do |c|
              next unless c.kind_of?(DynamicElement)
              cg.when_(c.name.intern.inspect) do
                cg.code("#{c.instance_name}.render_with(it, __binding__)")
              end
            end
          end
        end
        cg.code("end")
      end
    end
  end
end
