
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'


context "Repeat Filter" do
  include Amrita2::Runtime
  include Amrita2::Filters
  
  specify "with string" do
    t = Amrita2::TemplateText.new('<div><span am:src="aaa|Repeat[5]" /></div>')
    t.render_with(:aaa => "A").should_be_samexml_as '<div>AAAAA</div>'
  end
end

context "Default Filter" do
  include Amrita2
  include Amrita2::Runtime
  include Amrita2::Filters

  specify "with not nil data" do
    begin
      t = Amrita2::TemplateText.new('<div><span am:src="aaa|Default[123]">aaa</span></div>')
      t.test_with(:aaa => "xyz") do |result|
        result.should_be_samexml_as '<div>xyz</div>'
      end
    end
  end

  specify "with nil " do
    t = Amrita2::TemplateText.new('<span><span am:src="aaa|Default[123]">aaa</span></span>')
    t.test_with(:aaa => nil) do |result|
      result.should_be_samexml_as '123'
    end
  end

  specify "append filter in am:filter" do
    text = '<span><span am:src="aaa" am:filter="Default[789]">aaa</span></span>'
    t = Amrita2::TemplateText.new(text) 
    t.test_with(:aaa => nil) do |result|
      result.should_be_samexml_as '789'
    end
  end
  
  specify "append filter in ruby code" do
    text = '<span><span am:src="aaa">aaa</span></span>'
    t = Amrita2::TemplateText.new(text) do |e, src, filters|
      if src == "aaa"
        filters << Amrita2::Filters::Default[456]
      end
    end
    t.test_with(:aaa => nil) do |result|
      result.should_be_samexml_as '456'
    end
  end

end

context "Format Filter" do
  include Amrita2::Runtime
  include Amrita2::Filters

  specify "Integer" do
    t = Amrita2::TemplateText.new(%q[<span><span am:src="aaa|Format['(%s)']">aaa</span></span>])
    t.test_with(:aaa => 123) do |result|
      result.should_be_samexml_as '(123)'
    end
  end

  specify "with Default" do
    t = Amrita2::TemplateText.new(%q[<span><span am:src="aaa|Default[123] | Format['(%s)']">aaa</span></span>])
    t.render_with(:aaa => nil).should_be_samexml_as '(123)'
  end
end

context "with am:v" do
  include Amrita2::Runtime
  include Amrita2::Filters

  specify "Integer" do
    text = %q[<span><span am:v="aaa" am:filter="Format['(%s)']">aaa</span></span>]
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    aaa = 123
    t.test_with(binding) do |result|
      result.should_be_samexml_as '(123)'
    end
  end
end

context "NVar" do
  include Amrita2::Runtime
  include Amrita2::Filters

  specify " " do
    text = <<-END
    <span>
      <a am:src="aaa|NVar" 
         href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>
    </span>
    END

    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa=>Amrita2::Tuple['amrita', 'Amrita template library']) do |result|
      result.should_be_samexml_as '<a href="http://raa.ruby-lang.org/list.rhtml?name=amrita">[Amrita template library]</a>'
    end
  end
  
  specify "with a hash" do
    text = <<-END
    <span>
      <a am:src="aaa|NVar[:name, :description]" 
         href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>
    </span>
    END
    data = {
      :name => 'amrita',
      :description => 'Amrita template library'
    }
    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa=>data) do |result|
      result.should_be_samexml_as '<a href="http://raa.ruby-lang.org/list.rhtml?name=amrita">[Amrita template library]</a>'
    end
  end
  
  specify "with an array of hash" do
    text = <<-END
    <ul>
      <li am:src="aaa|NVar[:name, :description]">
         <a href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>
      </li>
    </ul>
    END
    data = [
      {
        :name => 'amrita',
        :description => 'Amrita template library'
      },
      {
        :name => 'amrita2',
        :description => 'Amrita2 template library'
      }
    ]
    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa=>data) do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>[Amrita template library]</a>
        </li>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>[Amrita2 template library]</a>
        </li>
      </ul>
      END
    end
  end
  
  specify "with an array of struct" do
    text = <<-END
    <ul>
      <li am:src="aaa|NVar[:name, :description]">
         <a href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>
      </li>
    </ul>
    END
    s = Struct.new(:name, :description)
    data = [
      s.new('amrita', 'Amrita template library'),
      s.new('amrita2', 'Amrita2 template library'),
    ]
    t = Amrita2::TemplateText.new(text)
    t.test_with(:aaa=>data) do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>[Amrita template library]</a>
        </li>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>[Amrita2 template library]</a>
        </li>
      </ul>
      END
    end
  end
end

