
require 'rexml/document'
require 'amrita2'
require 'amrita2/testsupport'

include Amrita2
include Amrita2::Filters
include Amrita2::Runtime


context "Inline Ruby" do
  include Amrita2::Runtime
  specify "set context value" do
    text = <<-END
    <div>
    <%
      $_ = { :aaa => "abc" + a }
    %>
    <span am:src="aaa">abc</span>
    </div>
    END
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    a = "xyz"
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>abcxyz</div>")
    end
  end
  
  specify "with cdata" do
    text = <<-'END'
    <div>
    <%
      $_ = { :aaa => SanitizedString["<em>#{a}</em>"] }
    %>
    <span am:src="aaa">abc</span>
    </div>
    END
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    a = "xyz"
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div><em>xyz</em></div>")
    end
  end
  
  specify "with an array of struct" do
    text = <<-'END'
    <ul>
      <li am:for="data" am:skipif='$_.name !~ /amrita/'>
        <%
          $_ = {
             :url => "http://raa.ruby-lang.org/list.rhtml?name=#{$_.name}",
             :description => ($_.description or $_.name)
          }
        %>
        <a am:filter='Attr[:href=>:url, :body=>:description]'/>
      </li>
    </ul>
    END
    s = Struct.new(:name, :description)
    data = [
      s.new('amrita', 'Amrita template library'),
      s.new('amrita2'),
      s.new('Ruby on Rails', 'this will be passed'),
    ]
    t = Amrita2::TemplateText.new(text)
    t.inline_ruby = true
    t.test_with(binding) do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>Amrita template library</a>
        </li>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>amrita2</a>
        </li>
      </ul>
      END
    end
  end
  
  specify "with erb" do
    text = <<-'END'
    <ul>
      <li am:for="data" am:skipif='$_.name !~ /amrita/'>
        <![CDATA[
          <%
            name = $_.name
            description = ($_.description or $_.name)
          %>
          <a href="http://raa.ruby-lang.org/list.rhtml?name=<%= name %>"><%= description %></a>
        ]]>
      </li>
    </ul>
    END
    s = Struct.new(:name, :description)
    data = [
      s.new('amrita', 'Amrita template library'),
      s.new('amrita2'),
      s.new('Ruby on Rails', 'this will be passed'),
    ]
    t = Amrita2::TemplateText.new(text)
    t.inline_ruby = true
    t.test_with(binding) do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>Amrita template library</a>
        </li>
        <li>
          <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>amrita2</a>
        </li>
      </ul>
      END
    end
  end
end

context "Logic IF" do
  include Amrita2::Runtime
  setup do
    @t = Amrita2::TemplateText.new('<span am:skipif="a == 1">abc</span>', :inline_ruby)
  end
  
  specify "if true" do
    a = 1
    @t.test_with(binding) do |result|
      result.should_be_samexml_as("")
    end
  end

  specify "false" do
    a = 0
    @t.test_with(binding) do |result|
      result.should_be_samexml_as("abc")
    end
  end
  
  specify "with loop" do
    text = <<-END
    <ul>
      <li am:for="1..5" am:skipif="($_ % 2) == 0" />
    </ul>
    END
    t = Amrita2::TemplateText.new(text, :inline_ruby)
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<ul><li>1</li><li>3</li><li>5</li></ul>")
    end
  end
end

context "Logic for" do
  include Amrita2::Runtime
  setup do
    @t = Amrita2::TemplateText.new('<div><span am:for="a" /></div>', :inline_ruby)
  end
  
  specify "0" do
    a = []
    @t.test_with(binding) do |result|
      result.should_be_samexml_as("<div></div>")
    end
  end
  specify "1" do
    a = [1]
    @t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>1</div>")
    end
  end

  specify "3" do
    a = [1, 2, 3]
    @t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>123</div>")
    end
  end
end

context "Logic value" do
  include Amrita2::Runtime
  specify "1" do
    t = Amrita2::TemplateText.new('<div><span am:v="a * 2" /></div>', :inline_ruby)
    a = 1
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>2</div>")
    end
  end

  specify "method call" do
    t = Amrita2::TemplateText.new('<div><span am:v="aaa(a)" /></div>', :inline_ruby)
    a = 123
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>246</div>")
    end
  end

  def aaa(x)
    x*2
  end
  
  specify "filter" do
    t = Amrita2::TemplateText.new('<div><span am:src="a" am:v="$_ * 2" /></div>')
    t.inline_ruby = true
    a = 1
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>2</div>")
    end
    a = "abc"
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>abcabc</div>")
    end
  end
  
  specify "quotation" do
    t = Amrita2::TemplateText.new(%[<div><span am:v=" 'abc' * a" /></div>], :inline_ruby)
    a = 2
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>abcabc</div>")
    end
    
    t = Amrita2::TemplateText.new(%[<div><span am:v=' "abc" * a' /></div>], :inline_ruby)
    a = 2
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>abcabc</div>")
    end
  end
  
  specify '#{ ...} in am:v' do
    t = Amrita2::TemplateText.new(%q[<div><span am:v=" %[#{a * 2}] " /></div>], :inline_ruby)
    a = 2
    t.test_with(binding) do |result|
      result.should_be_samexml_as("<div>4</div>")
    end
  end
end
