require 'rexml/document'
require 'amrita2'
require 'amrita2/macro'
require 'amrita2/testsupport'

include Amrita2::Runtime

context "Macro can expand child elements" do
  module MacroTest
    include Amrita2
    include Filters
    class Hello4 < Macro::Base
      include Amrita2
      TemplateText = 
        %[<p macro:src="greeting"><span macro:src="msg" />, macro world! ] +
        %[<span macro:src="content" /> </p>]

      def match_element(element)
        element.expanded_name == 'hello'
      end
      
      def preprocess_element(mt, element)
        mt.render_with(:greeting=>{
                         :msg => element.attributes["msg"],
                         :content => element.children
                       })
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
    <ul>
      <li><hello msg="Good morning"><span am:src="msg1" /><em am:src="msg2" /></hello></li>
      <li><hello msg="Hello"       ><strong am:src="msg3" /></hello></li>
    </ul>
    END
    t = Amrita2::TemplateText.new(text)
    t.add_macro(MacroTest::Hello4.new)
    t.test_with(:msg1 => "Try it.", :msg2=>"Enjoy it!", :msg3=> "It's great, isn't it?") do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li><p>Good morning, macro world!  Try it.<em>Enjoy it!</em> </p></li>
        <li><p>Hello, macro world! <strong>It's great, isn't it?</strong> </p></li>
      </ul>
      END
    end
  end
end

context "Macro can contain dynamic element" do
  module MacroTest
    include Amrita2
    include Filters
    class Hello3 < Macro::Base
      include Amrita2
      TemplateText = 
        %[<span macro:src="greeting" />, macro world! ] +
        %[<span macro:filter="Attr[:target_src=>:id]" />]

      def match_element(element)
        element.expanded_name == 'hello'
      end
      
      def preprocess_element(mt, element)
        mt.render_with(:greeting=>element.attributes["greeting"],
                       :id => element.attributes["amrita_id"])
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
    <ul>
      <li><hello greeting="Good morning" amrita_id="msg1"/></li>
      <li><hello greeting="Hello"        amrita_id="msg2"/></li>
    </ul>
    END
    t = Amrita2::TemplateText.new(text)
    t.add_macro(MacroTest::Hello3.new)
    t.test_with(:msg1 => "Enjoy it!", :msg2=> "It's great, isn't it?") do |result|
      result.should_be_samexml_as <<-END
      <ul>
        <li>Good morning, macro world! Enjoy it!</li>
        <li>Hello, macro world! It's great, isn't it?</li>
      </ul>
      END
    end
  end
end

context "Macro can contain dynamic element" do
  module MacroTest
    include Amrita2
    include Filters
    class Hello2 < Macro::Base
      include Amrita2
      TemplateText = <<-END.strip
<span macro:src="greeting" />, macro world!
<span target_src="msg" />
END

      def match_element(element)
        element.expanded_name == 'hello'
      end
      
      def preprocess_element(mt, element)
        mt.render_with(:greeting=>'Hello')
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
    <span><hello /> </span>
    END
    t = Amrita2::TemplateText.new(text)
    t.add_macro(MacroTest::Hello2.new)
    t.test_with(:msg => "Enjoy it!") do |result|
      result.should_be == "Hello, macro world!\nEnjoy it! \n"
    end
  end
end

context "Amrita2 has Macro" do
  module MacroTest
    include Amrita2
    include Filters
    class Hello < Macro::Base
      include Amrita2
      TemplateText = <<-END
        <span macro:src="greeting" />, macro world!
      END

      def match_element(element)
        element.expanded_name == 'hello'
      end
      
      def preprocess_element(mt, element)
        mt.render_with(:greeting=>'Hello')
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
    <span><hello /> <span am:src="msg" /></span>
    END
    t = Amrita2::TemplateText.new(text)
    t.add_macro(MacroTest::Hello.new)
    t.test_with(:msg => "Enjoy it!") do |result|
      result.should_be == "Hello, macro world!\n Enjoy it!\n"
    end
  end
end

context "Amazon Link Macro" do
  module MacroTest
    include Amrita2
    include Filters
    class Amazon < Macro::Base
      include Amrita2
      TemplateText = <<-END
      <span>
        <a macro:src = "book|NVar[:isbn, :content, :p1, :p2]"
           target_src = "book|NVar[:$1, :$2]"
           href="http://www.amazon.co.jp/exec/obidos/ASIN/$$1/">
          $$2
        </a>
      </span>
      END

      def match_element(element)
        element.expanded_name == 'book'
      end
      
      def preprocess_element(mt, element)
        a = element.attributes
        data = {
          :isbn => a["isbn"] || "isbn",
          :content => a["content"] || "content",
        }
        #mt.set_trace(STDOUT)
        mt.render_with(:book=>data)
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
    <div>
       <book isbn="isbn" content="title" />
    </div>
    END
#    
#   after macro expansion
#   <div><a href='http://www.amazon.co.jp/exec/obidos/ASIN/$1/'
#           am:src='book|NVar[:isbn_no, :title]'>$2</a></div>
#   
    t = Amrita2::TemplateText.new(text) do |e, name, filters|
      filters << Amrita2::Filters::MacroFilter[MacroTest::Amazon]
    end
    
    data = {
      :book=>{
        :isbn=>'4798013951',
        :title=>'Ruby on Rails for dummies'
      }
    }
    #t.set_trace(STDOUT)
    t.test_with(data) do |result|
      result.should_be_samexml_as <<-END
        <div>
          <a href="http://www.amazon.co.jp/exec/obidos/ASIN/4798013951/">Ruby on Rails for dummies</a>
        </div>
      END
    end
  end
end

context "Macro with erb" do
  module MacroTest
    include Amrita2
    include Filters
    class Formula < Macro::Base
      include Amrita2
      TemplateText = <<-END
      <span macro:src = "formula">
        <span macro:skipif="$_[:op] != '+'">
          <span macro:filter="NVar[:name, :term1, :term2]"
                target_src="$1">
             <%%= $_[:$2] %%> + <%%= $_[:$3] %%> = <%%= $_[:$2].to_i + $_[:$3].to_i %%>
          </span>
        </span>
        <span macro:skipif="$_[:op] != '-'">
          <span macro:filter="NVar[:name, :term1, :term2]"
                target_src="$1">
             <%%= $_[:$2] %%> - <%%= $_[:$3] %%> = <%%= $_[:$2].to_i - $_[:$3].to_i %%>
          </span>
        </span>
      </span>
      END

      def match_element(element)
        element.expanded_name == 'formula'
      end
      
      def preprocess_element(mt, element)
        a = element.attributes
        op = a["op"]
        t1 = a["term1"]
        t2 = a["term2"]
        name = a["name"]
        #mt.set_trace(STDOUT)
        mt.render_with(:formula=>{ :op=>op, :term1=>t1, :term2=>t2, :name=>name })
      end
    end
  end
  include MacroTest

  specify " " do
    text = <<-END
      <formula name="plus" op="+" term1="term1" term2="term2"/>/
      <formula name="minus" op="-" term1="num1" term2="num2"/>
    END

    t = Amrita2::TemplateText.new(text,:inline_ruby) do |e, name, filters|
      filters << Amrita2::Filters::MacroFilter[MacroTest::Formula]
    end
    
#    
#   after macro expansion
#    <span target_src='plus'>
#      <%%= $_[:term1] %> + <%%= $_[:term2] %> = <%%= $_[:term1].to_i + $_[:term2].to_i %>
#    </span>
#    <span target_src='minus'>
#      <%%= $_[:num1] %> - <%%= $_[:num2] %> = <%%= $_[:num1].to_i - $_[:num2].to_i %>
#    </span>
#   

    data = {
      :plus => { 
        :term1 => 10,
        :term2 => 20,
      },
      :minus => { 
        :num1 => 30,
        :num2 => 5
      }
    }
    #t.set_trace(STDOUT)
    t.test_with(data) do |result|
      r = result.gsub(/[\s\n]*/, "")
      r.should_be == '10+20=30/30-5=25'
    end
  end
end
